'use strict';

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

const statuses = require('statuses');

const names = {};
const messages = {};
const table = {};

/**
 * Generate code to error and name to error mapping.
 *
 * Error have three fields
 *   * name
 *   * message
 *   * status
 */
statuses.codes.forEach(code => {
  const prefix = String(code).charAt(0);
  if (prefix === '4' || prefix === '5') {
    const name = toIdentifier(statuses[code]);
    names[code] = name;
    messages[code] = statuses[code];
    table[name] = {
      name,
      message: statuses[code],
      status: code
    };
  }
});

// generate error classes

/**
 * Convert a string of words to a JavaScript identifier.
 * @private
 */
function toIdentifier(str) {
  return str.split(' ').map(function (token) {
    return token.slice(0, 1).toUpperCase() + token.slice(1);
  }).join('').replace(/[^ _0-9a-z]/gi, '');
}

let HttpError = function (_Error) {
  _inherits(HttpError, _Error);

  function HttpError(name, message) {
    _classCallCheck(this, HttpError);

    var _this = _possibleConstructorReturn(this, (HttpError.__proto__ || Object.getPrototypeOf(HttpError)).call(this, message));

    _this.name = name;
    // backward compability
    _this.type = name;
    return _this;
  }

  return HttpError;
}(Error);

let ParameterError = function (_HttpError) {
  _inherits(ParameterError, _HttpError);

  function ParameterError(field, expected, actual) {
    _classCallCheck(this, ParameterError);

    let message = `${ field } should be ${ expected }, got ${ actual }`;

    var _this2 = _possibleConstructorReturn(this, (ParameterError.__proto__ || Object.getPrototypeOf(ParameterError)).call(this, 'BadRequest', message));

    _this2.field = field;
    return _this2;
  }

  return ParameterError;
}(HttpError);

module.exports = {
  HttpError,
  ParameterError,
  names,
  messages,
  table
};