'use strict';

let createApplicationAsync = (() => {
  var _ref = _asyncToGenerator(function* (app) {
    let options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    try {
      var _options$logger = options.logger;
      _options$logger = _options$logger === undefined ? {} : _options$logger;
      const name = _options$logger.name,
            file = _options$logger.file,
            logstash = _options$logger.logstash,
            sentry = _options$logger.sentry;
      var _options$dynamicDefin = options.dynamicDefinition;
      const dynamicDefinition = _options$dynamicDefin === undefined ? {} : _options$dynamicDefin,
            errorHandler = options.errorHandler,
            model = options.model;


      logger.initialize({ name, file, logstash, sentry });

      var _ref2 = yield globals.initialize({ model });

      const raw = _ref2.raw,
            models = _ref2.models,
            services = _ref2.services,
            securitys = _ref2.securitys,
            middlewares = _ref2.middlewares;

      exports.app = app;
      exports.raw = raw;
      exports.models = models;
      exports.services = services;
      exports.securitys = securitys;
      exports.middlewares = exports.mws = middlewares;

      const api = yield blueprint.initialize(dynamicDefinition, models);
      exports.api = api;
      if (rc.env === 'test') {
        exports.test = require('./supertest').initialize(app, api);
      }

      app.use(function (req, res, next) {
        req.state = {
          version: api.definition.info.version
        };
        const operation = api.getOperation(req);
        req.swagger = {
          operation: operation
        };
        next();
      });

      //
      // Response pipeline
      //

      const pipeline = Pipeline();
      pipeline.use(mws.debug.postHandler());
      // remove response `null` fields
      pipeline.use(mws.validation.response());
      pipeline.use(mws.debug.response());
      app.use(pipeline.createMiddleware());

      //
      // Request pipeline
      //

      // binding request context
      app.use(mws.context(rc.target + '/context'));
      // setup swagger documentation
      app.use(mws.docs(api.definition));
      app.use(mws.switch());
      // request & response logging
      app.use(mws.debug.request());
      // request validation
      app.use(mws.validation.request());
      // security handler
      app.use(mws.security(api, securitys));
      // dynamic swagger defined route
      app.use(mws.debug.preHandler());
      app.use(api.basePathPrefix, mws.api(api));
      // 404
      app.use(api.basePathPrefix, function () {
        throw new HttpError('NotFound');
      });
      // error handling
      app.use(mws.errorHandler(api, errorHandler));

      return app;
    } catch (err) {
      console.log(err.stack); // eslint-disable-line
      process.exit(1);
    }
  });

  return function createApplicationAsync(_x) {
    return _ref.apply(this, arguments);
  };
})();

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const rc = require('./rc');
const utility = require('./utility');
const Pipeline = require('./pipeline');
const mws = require('./middleware');
const logger = require('./logger');
const globals = require('./globals');
const blueprint = require('./blueprint');

var _require = require('./swagger');

const parseAsync = _require.parseAsync;

var _require2 = require('./error');

const HttpError = _require2.HttpError,
      ParameterError = _require2.ParameterError;


exports.createApplicationAsync = createApplicationAsync;
// backward compability
exports.ResponseError = exports.HttpError = HttpError;
exports.ParameterError = ParameterError;
exports.translate = exports.utility = utility;
exports.parseAsync = parseAsync;
exports.logger = logger;
exports.rc = rc;
exports.wrapRoute = utility.wrapRoute;
exports.wrapMiddleware = utility.wrapMiddleware;

process.on('unhandledRejection', function (reason, p) {
  let err = new Error('Unhandled Promise rejection');
  err.reason = reason;
  err.promise = p;
  logger.error('Unhandled Rejection at: Promise ', p, ' reason: ', reason);
  logger.error(reason.message, reason.stack);
  throw err;
});