'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const crypto = require('crypto');

var _require = require('chai');

const expect = _require.expect;


describe('security', () => {
  it('should verify security parameters by yourself', _asyncToGenerator(function* () {
    var _ref2 = yield api.get('/security/basic').expect(400);

    const body = _ref2.body;

    expect(body.name).to.eql('BadRequest');
  }));

  it('should check if matching username and password', _asyncToGenerator(function* () {
    var _ref4 = yield api.get('/security/basic').auth('username', 'invalid password').expect(403);

    const body = _ref4.body;

    expect(body.name).to.eql('Forbidden');
  }));

  it('should return current user', _asyncToGenerator(function* () {
    var _ref6 = yield api.get('/security/basic').auth('username', md5('username')).expect(200);

    const body = _ref6.body;

    expect(body.username).to.eql('username');
  }));

  it('should fast return on first successful authentication', _asyncToGenerator(function* () {
    var _ref8 = yield api.get('/security/multiple').set('token', 'example-token').expect(200);

    const body = _ref8.body;

    expect(body.username).to.eql('username');
  }));

  it('should one failed authentication wont break request', _asyncToGenerator(function* () {
    var _ref10 = yield api.get('/security/multiple').set('token', 'invalid token').auth('username', md5('username')).expect(200);

    const body = _ref10.body;

    expect(body.username).to.eql('username');
  }));

  it('should return last authentication failure message', _asyncToGenerator(function* () {
    var _ref12 = yield api.get('/security/multiple').set('token', 'invalid token').auth('username', 'invalid password').expect(403);

    const body = _ref12.body;

    expect(body.name).to.eql('Forbidden');
    expect(body.message).to.eql('Invalid username or password');
  }));
});

function md5(str) {
  return crypto.createHash('md5').update(str, 'utf8').digest('hex');
}