'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var _require = require('chai');

const expect = _require.expect;


describe('validation', () => {
  let body;

  it('should validate required parameter', _asyncToGenerator(function* () {
    yield api.get('/validation?required=true').expect(200);

    var _ref2 = yield api.get('/validation').expect(400);

    body = _ref2.body;

    expect(body.name).to.eql('BadRequest');
    expect(body.message).to.contain('Invalid parameter (required): Value is required but was not provided');
  }));

  it('should validate string and number parameter', _asyncToGenerator(function* () {
    yield api.get('/validation?required=true&string=123&number=123').expect(200);
    yield api.get('/validation?required=true&string=char&number=123').expect(200);

    var _ref4 = yield api.get('/validation?required=true&string=123&number=char').expect(400);

    body = _ref4.body;

    expect(body.name).to.eql('BadRequest');
    expect(body.message).to.contain('Invalid parameter (number): Expected type number but found type string');
  }));

  it('should validate array parameter', _asyncToGenerator(function* () {
    yield api.get('/validation').query({ required: true, numberArray: [1, 2] }).expect(200);

    var _ref6 = yield api.get('/validation').query({ required: true, numberArray: [1, 'char'] }).expect(400);

    body = _ref6.body;

    expect(body.name).to.eql('BadRequest');
    expect(body.message).to.contain('Invalid parameter (numberArray): Expected type integer but found type string');

    yield api.get('/validation').query({ required: true, numberArray: ['123'] }).expect(200);
  }));

  it('should validate boolean parameter', _asyncToGenerator(function* () {
    yield api.get('/validation?required=true&boolean=true').expect(200);
    yield api.get('/validation?required=true&boolean=false').expect(200);

    // number are also treated as string in querystring

    var _ref8 = yield api.get('/validation?required=true&boolean=123').expect(400);

    body = _ref8.body;

    expect(body.name).to.eql('BadRequest');
    expect(body.message).to.contain('Invalid parameter (boolean): Expected type boolean but found type string');

    var _ref9 = yield api.get('/validation?required=true&boolean=char').expect(400);

    body = _ref9.body;

    expect(body.name).to.eql('BadRequest');
    expect(body.message).to.contain('Invalid parameter (boolean): Expected type boolean but found type string');
  }));

  it('should validate response', _asyncToGenerator(function* () {
    yield api.post('/validation').type('json').send({ message: 'ok' }).expect(200);

    var _ref11 = yield api.post('/validation').type('json').send({ not_message: 'ok' }).expect(500);

    body = _ref11.body;

    expect(body.name).to.eql('InternalServerError');
  }));
});