'use strict';

let parseAsync = (() => {
  var _ref = _asyncToGenerator(function* () {
    var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        _ref2$root = _ref2.root;

    let root = _ref2$root === undefined ? rc.target : _ref2$root;
    var _ref2$dynamicDefiniti = _ref2.dynamicDefinition;
    let dynamicDefinition = _ref2$dynamicDefiniti === undefined ? {} : _ref2$dynamicDefiniti;

    const definition = YAML.load(fs.readFileSync(root + '/spec/swagger.yaml'));
    definition.paths = glob.sync(`**/*${ rc.suffix.spec }`, { cwd: root }).sort().reduce(function (paths, file) {
      const location = path.join(root, file);
      const yaml = YAML.load(fs.readFileSync(location, 'utf8'));
      if (rc.shadow) {
        _.assign(paths, yaml);
      } else {
        const name = utility.fromSpec(file);
        paths[name] = yaml;
      }
      return paths;
    }, {});
    ['definitions', 'parameters'].forEach(function (target) {
      definition[target] = loadFiles(`spec/${ target }/*.yaml`, root);
    });
    definition['x-pages'] = loadFiles('spec/pages/*.md', root);
    if (isExist(root + '/spec/errors.yaml')) {
      definition['x-errors'] = YAML.load(fs.readFileSync(root + '/spec/errors.yaml'));
    }

    // merge rc.swagger and swagger.yaml
    _.merge(definition, dynamicDefinition);

    return definition;
  });

  return function parseAsync() {
    return _ref.apply(this, arguments);
  };
})();

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const fs = require('fs');
const path = require('path');

const YAML = require('js-yaml');
const glob = require('glob');
const _ = require('lodash');

const rc = require('../rc');
const utility = require('../utility');

module.exports = parseAsync;

function loadFiles(pattern, root) {
  return glob.sync(pattern, { cwd: root }).reduce((defs, file) => {
    const location = path.join(root, file);
    const name = path.parse(file).name;
    defs[name] = fs.readFileSync(location, 'utf8');
    if (path.extname(file) === '.yaml') {
      defs[name] = YAML.load(defs[name]);
    }
    return defs;
  }, {});
}

function isExist(file) {
  try {
    fs.accessSync(file);
    return true;
  } catch (err) {
    return false;
  }
}