'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var _require = require('chai');

const expect = _require.expect;

const sinon = require('sinon');

const utility = require('./utility');

describe('utility', () => {
  it('should generator or async middleware call subsequence middleware', _asyncToGenerator(function* () {
    let next;
    let middleware;

    const generator = function* (req, res) {
      return yield delay(1);
    };
    next = sinon.spy();
    middleware = utility.wrapMiddleware(generator);
    middleware({}, {}, next);
    yield delay(10);
    expect(next).to.be.called;

    const asyncawait = (() => {
      var _ref2 = _asyncToGenerator(function* (req, res) {
        return yield delay(1);
      });

      return function asyncawait(_x, _x2) {
        return _ref2.apply(this, arguments);
      };
    })();
    next = sinon.spy();
    middleware = utility.wrapMiddleware(asyncawait);
    middleware({}, {}, next);
    yield delay(10);
    expect(next).to.be.called;
  }));

  it('should generator or async router dont call next', _asyncToGenerator(function* () {
    let next;
    let middleware;

    const generator = function* (req, res) {
      return yield delay(1);
    };
    next = sinon.spy();
    middleware = utility.wrapRoute(generator);
    middleware({}, {}, next);
    yield delay(10);
    expect(next).to.not.be.called;

    const asyncawait = (() => {
      var _ref4 = _asyncToGenerator(function* (req, res) {
        return yield delay(1);
      });

      return function asyncawait(_x3, _x4) {
        return _ref4.apply(this, arguments);
      };
    })();
    next = sinon.spy();
    middleware = utility.wrapRoute(asyncawait);
    middleware({}, {}, next);
    yield delay(10);
    expect(next).to.not.be.called;
  }));

  it('should utility from uri to location', () => {
    expect(utility.toCode('/demo/hello')).to.eql('api/demo/hello.code.js');
    expect(utility.toTest('/demo/hello')).to.eql('api/demo/hello.test.js');
    expect(utility.toSpec('/demo/hello')).to.eql('api/demo/hello.spec.yaml');
  });
  it('should utility from location to uri', () => {
    expect(utility.fromCode('api/demo/hello.code.js')).to.eql('/demo/hello');
    expect(utility.fromTest('api/demo/hello.test.js')).to.eql('/demo/hello');
    expect(utility.fromSpec('api/demo/hello.spec.yaml')).to.eql('/demo/hello');
  });
  it('should detect location pattern', () => {
    expect(utility.isCode('api/demo/hello.code.js')).to.eql(true);
    expect(utility.isTest('api/demo/hello.test.js')).to.eql(true);
    expect(utility.isSpec('api/demo/hello.spec.yaml')).to.eql(true);

    expect(utility.isCode('api/demo/hello.test.js')).to.eql(false);
    expect(utility.isCode('api/demo/hello.code.yaml')).to.eql(false);
  });
});

function delay(timeout) {
  return new Promise(resolve => {
    setTimeout(() => {
      resolve(timeout);
    }, timeout);
  });
}